#ifdef _WIN32   // Windows
#include "windows.h"
#else           // Mac
#define _MAC 1
#define HANDLE Handle
#include <Memory.h>
#include <Scrap.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
#include "mm_jsapi.h"
#ifdef __cplusplus
}
#endif

// Define the forward function prototypes
JSBool
setClipboardContent( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval );

JSBool
getClipboardContent( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval );

JSBool
getClipboardContentSize( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval );

void MM_Init( void );

MM_STATE

void
MM_Init( void )
{
   JS_DefineFunction("setClipboardContent", setClipboardContent, 1 );
   JS_DefineFunction("getClipboardContent", getClipboardContent, 0 );
   JS_DefineFunction("getClipboardContentSize", getClipboardContentSize, 0);
}

JSBool setClipboardContent( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval )
{
   char *clipText;
   unsigned int clipTextLen;
   HANDLE hMem;

   // Get the first argument
   clipText = JS_ValueToString(cx, jsArgv[0], &clipTextLen);
if ( clipText == NULL) {
      *rval = JS_IntegerToValue( 0 );
      return JS_TRUE;
   }

#ifdef _WIN32
   LPVOID pText;

   if (::OpenClipboard(NULL))
   {
      ::EmptyClipboard();

      // Create the memory handle
      hMem = ::GlobalAlloc( GMEM_MOVEABLE + GMEM_DDESHARE + GMEM_ZEROINIT, clipTextLen);
      pText = ::GlobalLock(hMem);

      memcpy(pText,clipText,(size_t)clipTextLen);
      ::GlobalUnlock(hMem);

      // Set the clipboard contents
      ::SetClipboardData(CF_TEXT, hMem);

      ::CloseClipboard();
      *rval = JS_IntegerToValue( 1 );
   }
   *rval = JS_IntegerToValue( 0 );
#else
   ::ZeroScrap();
   ::PutScrap(clipTextLen,'TEXT',clipText);
   
   *rval = JS_IntegerToValue( 1 );
#endif
   return JS_TRUE;
}

JSBool getClipboardContent( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval )
{
   HANDLE hMem;
   char *pText = NULL;
   unsigned int iTextSize = 0;

#ifdef _WIN32
   // get the clipboard text if there is any
   if (::OpenClipboard(NULL))
   {
      if (::IsClipboardFormatAvailable(CF_TEXT))
      {
         // get the string data
         hMem = ::GetClipboardData(CF_TEXT);
         pText = (char *)::GlobalLock(hMem);
         iTextSize = strlen((const char *)pText);
         JS_StringToValue(cx,pText,iTextSize,rval);
         ::GlobalUnlock(hMem);
      }
      else
         *rval = (jsval)NULL;

      ::CloseClipboard();
   }
   else
      *rval = (jsval)NULL;
#else
   long offset;

   iTextSize = ::GetScrap(NULL, 'TEXT', &offset);
   hMem = NewHandleClear(iTextSize);
   ::GetScrap(hMem, 'TEXT', &offset);
   HLock(hMem);
   JS_StringToValue( cx, *hMem, iTextSize, rval );
   HUnlock(hMem);
   DisposeHandle(hMem);
#endif
   return JS_TRUE;
}

JSBool getClipboardContentSize( JSContext* cx, JSObject* obj, unsigned int  argc, jsval* jsArgv, jsval* rval )
{
   HANDLE hMem;
   char *pText;
   unsigned int iTextSize = -1;

#ifdef _WIN32
   if (::OpenClipboard(NULL))
   {
      if (::IsClipboardFormatAvailable(CF_TEXT))
      {
         // get the string data
         hMem = ::GetClipboardData(CF_TEXT);
         pText = (char *)::GlobalLock(hMem);
         iTextSize = strlen((const char *)pText);
         ::GlobalUnlock(hMem);
      }

      ::CloseClipboard();
   }
#else
   long offset;
   
   iTextSize = ::GetScrap(NULL, 'TEXT', &offset);
#endif

   *rval = JS_IntegerToValue(iTextSize);

   return JS_TRUE;
}
